// File edited 02/20/2015 with four space tab spacing

/** \file sersendf.c
	\brief Simplified printf implementation
*/
#include "sersendf.h"
#include <stdarg.h>
#include "simpletext.h"
#include "sermsg.h"

/** \brief Simplified printf
	\param format pointer to output format specifier string stored in FLASH.
	\param ... output data

	Implements only a tiny subset of printf's format specifiers :-

	%[ls][udcx%]

	l - following data is (32 bits)\n
	s - following data is short (8 bits)\n
	none - following data is 16 bits.

	u - unsigned int\n
	d - signed int\n
	q - signed int with decimal before the third digit from the right\n
	c - character\n
	x - hex\n
	% - send a literal % character

	Example:

	\code sersendf_P(PSTR("X:%ld Y:%ld temp:%u.%d flags:%sx Q%su/%su%c\n"), target.X, target.Y, current_temp >> 2, (current_temp & 3) * 25, dda.allflags, mb_head, mb_tail, (queue_full()?'F':(queue_empty()?'E':' '))) \endcode
*/

void sersendf_P(PGM_P format, ...)
{
	va_list args;
	va_start(args, format);

	uint16_t i = 0;
	uint8_t c = 1;
	uint8_t j = 0;
 
	while((c = pgm_read_byte(&format[i++])))
	{
		if(j)
		{
			switch(c)
			{
				case 's':

					j = 1;

					break;

				case 'l':

					j = 4;

					break;

				case 'u':    

					if(j == 4)
					{
						serwrite_uint32((uint32_t)va_arg(args, uint32_t));
					}
					else
					{
						serwrite_uint16((uint16_t)va_arg(args, uint32_t));
					}

					j = 0;

					break;

				case 'd':
    
					if(j == 4)
					{
						serwrite_int32((int32_t)va_arg(args, int32_t));
					}
					else
					{
						serwrite_int16((int16_t)va_arg(args, int32_t));
					}

					j = 0;

					break;

				case 'c':
    
					putChar((uint16_t)va_arg(args, uint32_t));

					j = 0;

					break;

				case 'x':

					putStr(PSTR("0x"));

					if(j == 4)
					{
						serwrite_hex32((uint32_t)va_arg(args, uint32_t));
					}
					else if(j == 1)
					{
						serwrite_hex8((uint8_t)va_arg(args, uint32_t));
					}
					else
					{
						serwrite_hex16((uint16_t)va_arg(args, uint32_t));
					}

					j = 0;

					break;

				// An original comment
				//case 'p':

					//serwrite_hex16(GET_ARG(uint16_t));

				case 'q':

					serwrite_int32_vf((int32_t)va_arg(args, int32_t), 3);

					j = 0;

					break;

				default:

					putChar(c);

					j = 0;

					break;
			}
		}
		
		else
		{

			if(c == '%')
			{
				j = 2;
			}
			else
			{
				putChar(c);
			}
		}
	}

	va_end(args);
}